% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Dr. Jos Ernesto Rayas-Snchez                                 April 10, 2019
% Research Group on Computer-Aided Engineering of Circuits and Systems (CAECAS)
% Department of Electronics, Systems and Informatics                      ITESO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%     Ideal RLC Parallel Lumped Resonator Simulated in Matlab (coarse model)
%
% This function calculates the magnitude and phase of S11 of a parallel 
% RLC lumped resonator. A reference impedance of 50 ohms is used.
%
% Usage: [psi,R] = RLCc_Matlab(P)                                         
%        P(1) = Rc; Parallel resonator resistor (ohms).
%        P(2) = Lc; Parallel resonator inductor (nH).
%        P(3) = Cc; Parallel resonator capacitor (pF).
%        P(4) = IF; Initial frequency (GHz).
%        P(5) = FF; Final frequency (GHz).
%        P(6) = FP; Number of frequency points.
%        psi: column vector containing the simulated frequency values (GHz).                    
%        R: matrix containing the circuit response values (S-parameters). 

function [psi,R] = RLCc_Matlab(P) 

% Reading Parameters
Rc = P(1);    % Parallel resonator resistor (ohms).
Lc = P(2);    % Parallel resonator inductor (nH).
Cc = P(3);    % Parallel resonator capacitor (pF).
IF = P(4);    % Initial frequency (GHz).
FF = P(5);    % Final frequency (GHz).
FP = P(6);    % Number of frequency samples.

% Calculating Responses
j = sqrt(-1);
Zo = 50;  % Reference impedance for the S parameters.
f = linspace(IF,FF,FP); % Row vector of frequencies (GHz).
w = 2*pi*1e9*f; % Angular frequency (rad/s).
Yin = 1/Rc + 1./(j*1e-9*Lc*w) + j*1e-12*Cc*w;
Zin = 1./Yin;
S11cx = (Zin-Zo)./(Zin+Zo);
mS11 = abs(S11cx);
pS11 = (180/pi)*angle(S11cx);

psi = f'; 
R(:,1) = mS11';
R(:,2) = pS11';
