% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Dr. Jos Ernesto Rayas-Snchez                                 April 10, 2019
% Research Group on Computer-Aided Engineering of Circuits and Systems (CAECAS)
% Department of Electronics, Systems and Informatics                      ITESO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%   Non-ideal RLC Parallel Lumped Resonator Simulated in Matlab (fine model)
%
% This function calculates the magnitude and phase of S11 of a parallel 
% RLC lumped resonator with some parasitics. A reference impedance of 50 ohms 
% is used.
%
% Usage: [psi,R] = RLCf_Matlab(P)                                         
%        P(1) = Rp; Parasitic resistor in series with inductor Lf (ohms).
%        P(2) = Lp; Parasitic inductor in series with capacitor Cf (nH).
%        P(3) = Rf; Parallel resonator resistor (ohms).
%        P(4) = Lf; Parallel resonator inductor (nH).
%        P(5) = Cf; Parallel resonator capacitor (pF).
%        P(6) = IF; Initial frequency (GHz).
%        P(7) = FF; Final frequency (GHz).
%        P(8) = FP; Number of frequency points.
%        psi: column vector containing the simulated frequency values (GHz).                    
%        R: matrix containing the circuit response values (S-parameters). 

function [psi,R] = RLCf_Matlab(P) 

% Reading Parameters
Rp = P(1);    % Parasitic resistor in series with inductor Lf (ohms).
Lp = P(2);    % Parasitic inductor in series with capacitor Cf (nH).
Rf = P(3);    % Parallel resonator resistor (ohms).
Lf = P(4);    % Parallel resonator inductor (nH).
Cf = P(5);    % Parallel resonator capacitor (pF).
IF = P(6);    % Initial frequency (GHz).
FF = P(7);    % Final frequency (GHz).
FP = P(8);    % Number of frequency samples.

% Calculating Responses
j = sqrt(-1);
Zo = 50;  % Reference impedance for the S parameters.
f = linspace(IF,FF,FP); % Row vector of frequencies (GHz).
w = 2*pi*1e9*f; % Angular frequency (rad/s).
Yin = 1/Rf + 1./(Rp+j*1e-9*Lf*w) + (j*1e-12*Cf*w)./(1-1e-9*Lp*1e-12*Cf*w.*w);
Zin = 1./Yin;
S11cx = (Zin-Zo)./(Zin+Zo);
mS11 = abs(S11cx);
pS11 = (180/pi)*angle(S11cx);

psi = f'; 
R(:,1) = mS11';
R(:,2) = pS11';
